/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef __AIDGE_OPERATORIMPL_H__
#define __AIDGE_OPERATORIMPL_H__

#include <cstddef>
#include <vector>
#include "utils/Types.h"

namespace Aidge {
class OperatorImpl {
public:
    virtual void forward(){};
    virtual void backward() {}

    /**
     * @brief Minimum amount of data from a specific input required by the
     * implementation to be run.
     *
     * @param inputIdx Index of the input analysed.
     * @return std::size_t
     */
    virtual NbElts_t getNbRequiredData(IOIndex_t inputIdx) const = 0;

    // Amount of input data that cannot be overwritten during the execution.
    virtual NbElts_t getNbRequiredProtected(IOIndex_t inputIdx) const = 0;

    // Memory required at an output for a given input size.
    virtual NbElts_t getRequiredMemory(IOIndex_t outputIdx, const std::vector<DimSize_t> &inputsSize) const = 0;

    /**
     * @brief Total amount of consumed data from a specific input.
     *
     * @param inputIdx Index of the input analysed.
     * @return DimSize_t
     */
    virtual NbElts_t getNbConsumedData(IOIndex_t inputIdx) const = 0;

    /**
     * @brief TOtal amount of produced data ready to be used on a specific output.
     *
     * @param outputIdx Index of the output analysed.
     * @return DimSize_t
     */
    virtual NbElts_t getNbProducedData(IOIndex_t outputIdx) const = 0;

    virtual ~OperatorImpl() = default;
};
} // namespace Aidge

#endif /* __AIDGE_OPERATORIMPL_H__ */
