/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef __AIDGE_SEQSTM_H__
#define __AIDGE_SEQSTM_H__

#include <iostream>
#include <map>
#include <regex>
#include <set>
#include <stdexcept> // for exception, runtime_error, out_of_range
#include <string>
#include <utility>
#include <vector>


#include "graphmatching/NodeRegex.hpp"
#include "graphmatching/Utile.hpp"


namespace Aidge {

class SeqStm {

private:
  const int mStmIdx;
  const std::vector<std::vector<int>> mTransitionMatrix;
  // str key of type like 'A' that ce use in the A->B .. extpr
  const std::map<std::string, NodeRegex *> mNodesRegex;
  // mTypeToIdxTransition.first = std::pair node_type , common_tag
  // mTypeToIdxTransition.segond = idx in trans matrix
  const std::map<NodeTypeKey, int> mTypeToIdxTransition;

  int mActSt;
  std::set<NodeTmp> mAllNodeValidated;
  std::set<NodeTmp> mAllNodeTested;
  std::set<std::pair<NodeTmp, std::string>> mAllCommonNode;
  bool mStmIsValid;

  std::pair<NodeRegex *, std::string> getNodeRegexAndCommonAt(int idxType);

  /**
   * @brief test the stm on a type
   * @return the common tag
   */
  std::string transitionOnNodeType(NodeType nodeType);

public:
  SeqStm(const int mStmIdx,
         const std::vector<std::vector<int>> &mTransitionMatrix,
         const std::map<std::string, NodeRegex *> &mNodesRegex,
         const std::map<NodeTypeKey, int> &mTypeToIdxTransition, int mActSt,
         std::set<NodeTmp> mAllNodeValidated, std::set<NodeTmp> mAllNodeTested,
         std::set<std::pair<NodeTmp, std::string>> mAllCommonNode,
         bool mStmIsValid);

  //////////////////////////////////////
  // STM test
  /////////////////////////////////////

  /**
   * @brief get if a st is a valide one
   * @return bool
   */
  bool isAValidSt(int st) {
    std::size_t size = mTransitionMatrix.size();
    return st == static_cast<int>(size - 1) ? true : false;
  }

  /**
   * @brief true if the stm is blocked into st
   * @return bool
   */
  bool isStmBlocked() { return mActSt == -1 ? true : false; }

  /**
   * @brief true if the stm into valide st
   * @return bool
   */
  bool isValid() { return mStmIsValid; }

  /////////////////////////////////////
  // utile
  /////////////////////////////////////
  /**
   * @brief extract from a node is type
   * @return bool
   */
  NodeType getTheNodeType(NodeTmp node);

  void drawStm();
  /////////////////////////////////////
  // geter
  /////////////////////////////////////

  std::set<std::pair<NodeTmp, std::string>> getAllCommonNode() {
    return mAllCommonNode;
  }
  std::set<NodeTmp> getAllNodeTested() { return mAllNodeTested; }

  std::set<NodeTmp> getAllNodeValidated() { return mAllNodeValidated; }

  SeqStm *duplicateStm();

  int getStmIdx() { return mStmIdx; }

  int getState() { return mActSt; }
  //////////////////////////////////////////
  // USE
  //////////////////////////////////////////
  /**
   * @brief test the stm on a node
   * @return  pair new stm state, the common tag
   */
  std::pair<int, std::string> testNode(const NodeTmp node);
};
} // namespace Aidge

#endif /* __AIDGE_SEQSTM_H__ */