


#ifndef _AIDGE_CONDITIONAL_PARSER_H_
#define _AIDGE_CONDITIONAL_PARSER_H_

#include "nodeTester/ConditionalLexer.hpp"
#include <memory> // for shared_ptr

namespace Aidge{

const std::map<ConditionalTokenTypes, std::size_t> ConditionalPrec{
    {ConditionalTokenTypes::AND,2},
    {ConditionalTokenTypes::OR,1}
};

    
class AstConditionalNode; // Forward declaration
using ASTNodeCh = std::vector<std::shared_ptr<AstConditionalNode>>;

/**
 * @brief this class is the node of an AST tree, this
 */
class AstConditionalNode: public std::enable_shared_from_this<AstConditionalNode>
{
    public:
    AstConditionalNode(std::shared_ptr<ConditionalToken> token,ASTNodeCh child ={});
    /**
     * @brief get the lexeme of the token
     * @return the lexeme value 
     */
    std::string getValue() const;
    /**
     * @brief get the type of the token
     * @return the type
     */
    ConditionalTokenTypes getType() const;
        std::string getValue() const;
    /**
     * @brief get the child of the node
     * @return child
     */
    const ASTNodeCh& getChilds() const ;
    /**
     * @brief test if the node is a leaf in the tree
     * @return true if a leaf 
     */
    bool isLeaf() const ;

    /**
     * @brief get the number of child
     * @return the number of child
     */
    std::size_t nbChild() const;
    private:
    /**
     * @brief the token of the node
     */
    const std::shared_ptr<ConditionalToken> mToken;
    /**
     * @brief list of child
     */
    const ASTNodeCh mChild;
};

/**
 * @brief this class uses the lexer to create an AST according to a set of gramer rules 
 */
class ConditionalParser{
    
    public:
    /**
     * @brief AST graph creation function 
     * @param ConditionalExpressions String representing the logical fuction to be performed
     */
    ConditionalParser(const std::string ConditionalExpressions);

    /**
     * @brief AST graph creation function 
     * @return The AST tree 
     */
    std::shared_ptr<AstConditionalNode> parse(void);


    private:
    /**
     * @brief restart at the start of the ConditionalExpressions for LEXER and restart  mCurrentToken
     */
    void rstParser(void);
    
    //////////////////

    /**
     * @defgroup ParsingFunctions Function for creating AST
     * @brief Functions for recursive construction of the AST representing grammar rules
     */

    /**
     * @ingroup ParsingFunctions
     * @brief Token reading and verification function  
     * 
     */
    void ackToken(ConditionalTokenTypes  tokenType);

    /**
     * @ingroup ParsingFunctions
     * @brief Function of grammar rules for values : (KEY|INTEGER|FOAT|STRING|LAMBDA lambda)
     * @return AST node 
     */
    std::shared_ptr<AstConditionalNode> constructAstVal(void);
    /**
    * @ingroup ParsingFunctions
    * @brief Function of grammar rules for comparison : val (EQ|NEQ) val | LPAREN expr RPAREN
    * @return AST node 
    */
    std::shared_ptr<AstConditionalNode> constructAstCmpr(void);
    /**
    * @ingroup ParsingFunctions
    * @brief Function of grammar rules for arguments of a lambda : LAMBDA val (ARGSEP val)* RPAREN
    * @return AST node 
    */
    std::shared_ptr<AstConditionalNode> constructAstLambda(void);
    /**
    * @ingroup ParsingFunctions
    * @brief Function of grammar rules for a expresion : cmpr ((AND | OR) cmpr)*
    * @return AST node 
    */
    std::shared_ptr<AstConditionalNode> constructAstExpr(std::size_t precLimit = 0);


    /**
    * @brief The actual token in the parce
    */
    std::shared_ptr<ConditionalToken> mCurrentToken;
    /**
    * @brief The lexem use
    */
    ConditionalLexer mLexer;

};


}

#endif //_AIDGE_CONDITIONAL_PARSER_H_
