/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef __AIDGE_REGISTRAR_H__
#define __AIDGE_REGISTRAR_H__

#ifdef PYBIND
#include <pybind11/pybind11.h>
#endif

#include <iostream>
#include <functional>
#include <map>
#include <cassert>

namespace Aidge {
#ifdef PYBIND
namespace py = pybind11;
#endif

template <class Key, class Func>
class Registrable {
public:
    typedef Key registrar_key;
    typedef std::function<Func> registrar_type;

    static std::map<Key, std::function<Func>>& registry()
    {
        #ifdef PYBIND
        if (std::getenv("AIDGE_CORE_WITH_PYBIND")){
            std::string name = std::string("registrar_")+typeid(Registrable<Key, Func>).name();
            static auto shared_data = reinterpret_cast<std::map<Key, std::function<Func>> *>(py::get_shared_data(name));
            if (!shared_data)
                shared_data = static_cast<std::map<Key, std::function<Func>> *>(py::set_shared_data(name, new std::map<Key, std::function<Func>>()));
            return *shared_data;
        }
        #endif // PYBIND
        static std::map<Key, std::function<Func>> rMap;
        return rMap;
    }

};

template <class C>
struct Registrar {
    Registrar(const typename C::registrar_key& key, typename C::registrar_type func) {
        //printf("REGISTRAR: %s\n", key.c_str());
        bool newInsert;
        std::tie(std::ignore, newInsert) = C::registry().insert(std::make_pair(key, func));
        //assert(newInsert && "registrar already exists");
    }

    static auto create(const typename C::registrar_key& key){
        const auto it = C::registry().find(key);
        assert(it != C::registry().end() && "invalid registrar key");

        return (*it).second;
    }
    static std::vector<typename C::registrar_key> getKeys(){
        std::vector<typename C::registrar_key> keys;
        for(auto keyValue : C::registry())
            keys.push_back(keyValue.first);
        return keys;
    }    
};
}


#endif // __AIDGE_REGISTRAR_H__