/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <memory>
#include <string>
#include "graph/GraphView.hpp"
#include "graph/Node.hpp"
#include "utils/Types.h"
#include "data/Data.hpp"

namespace py = pybind11;
namespace Aidge {
void init_GraphView(py::module& m) {
    py::class_<GraphView, std::shared_ptr<GraphView>>(m, "GraphView")
            .def(py::init<>())
            .def("save", &GraphView::save, py::arg("path"), py::arg("verbose") = false)
            .def("get_output_nodes", &GraphView::outputNodes)
            .def("get_input_nodes", &GraphView::inputNodes)
            .def("add", (void (GraphView::*)(std::shared_ptr<Node>, bool)) & GraphView::add,
                 py::arg("other_node"), py::arg("include_learnable_parameters") = true)
            .def("add_child",
                 (void (GraphView::*)(std::shared_ptr<Node>,
                                      std::shared_ptr<Node>,
                                      const IOIndex_t,
                                      IOIndex_t)) &
                         GraphView::addChild,
                 py::arg("toOtherNode"), py::arg("fromOutNode") = nullptr,
                 py::arg("fromTensor") = 0U, py::arg("toTensor") = gk_IODefaultIndex)
            .def("replace_with", &GraphView::replaceWith, py::arg("new_nodes"))
            .def("get_nodes", &GraphView::getNodes)
            .def("get_node", &GraphView::getNode, py::arg("node_name"))
            .def("forward_dims", &GraphView::forwardDims)
            .def("__call__", &GraphView::operator(), py::arg("connectors"))
            .def("set_datatype", &GraphView::setDatatype, py::arg("datatype"))
            .def("set_backend", &GraphView::setBackend, py::arg("backend"))
          //   .def("__getitem__", [](Tensor& b, size_t idx)-> py::object {
          //      // TODO : Should return error if backend not compatible with get
          //      if (idx >= b.size()) throw py::index_error();
          //      switch(b.dataType()){
          //           case DataType::Float32:
          //                return py::cast(static_cast<float*>(b.getImpl()->rawPtr())[idx]);
          //           case DataType::Int32:
          //                return py::cast(static_cast<int*>(b.getImpl()->rawPtr())[idx]);
          //           default:
          //                return py::none();
          //           }
          //      })
            ;
}
}  // namespace Aidge
