/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include <memory>

#include "graph/GraphView.hpp"
#include "graph/Node.hpp"
#include "utils/Types.h"

namespace py = pybind11;
namespace Aidge {
void init_Node(py::module& m) {
    py::class_<Node, std::shared_ptr<Node>>(m, "Node")
            .def("name", &Node::name)
            .def("type", &Node::type)
            .def("get_operator", &Node::getOperator)
            .def("set_name", &Node::setName, py::arg("name"))
            .def("add_child",
                 (void (Node::*)(std::shared_ptr<Node>, const IOIndex_t, IOIndex_t)) &
                         Node::addChild,
                 py::arg("other_node"), py::arg("out_id") = 0, py::arg("other_in_id") = -1)
            .def("add_child",
                 (void (Node::*)(std::shared_ptr<GraphView>, const IOIndex_t,
                                 std::pair<std::shared_ptr<Node>, IOIndex_t>)) &
                         Node::addChild,
                 py::arg("other_graph"), py::arg("out_id") = 0,
                 py::arg("other_in_id") =
                         std::pair<std::shared_ptr<Node>, IOIndex_t>(nullptr, gk_IODefaultIndex))
            .def("inputs", &Node::inputs)
            .def("input", &Node::input, py::arg("inID"))
            .def("outputs", &Node::outputs)
            .def("output", &Node::output, py::arg("outID"))
            .def("get_nb_inputs", &Node::nbInputs)
            .def("get_nb_datainputs", &Node::nbDataInputs)
            .def("get_nb_outputs", &Node::nbOutputs)
            .def("__call__", &Node::operator(), py::arg("connectors"));
}
}  // namespace Aidge
