/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include "utils/Types.h"
#include "utils/Parameter.hpp"
// #include "backend/OperatorImpl.hpp>
#include "operator/Operator.hpp"
#include "operator/Producer.hpp"

namespace py = pybind11;
namespace Aidge {

template <DimIdx_t DIM>
void declare_Producer(py::module &m) {
    py::class_<Producer_Op<DIM>,  std::shared_ptr<Producer_Op<DIM>>, Operator>(
        m, 
        ("ProducerOp" + std::to_string(DIM)+"D").c_str(), 
        py::multiple_inheritance())
    .def("set_output", &Producer_Op<DIM>::setOutput, py::arg("output_data"));
    // m.def(("Producer_" + std::to_string(DIM)+"D").c_str(), py::overload_cast<shared_ptr<Node>&>(&Producer<DIM>), py::arg("dims"), py::arg("name"));
    m.def("Producer", static_cast<std::shared_ptr<Node>(*)(const std::array<DimSize_t, DIM>&, const char*)>(&Producer), py::arg("dims"), py::arg("name"));
    
}


void init_Producer(py::module &m) {
    declare_Producer<1>(m);
    declare_Producer<2>(m);
    declare_Producer<3>(m);
    declare_Producer<4>(m);
    declare_Producer<5>(m);
    declare_Producer<6>(m);
    
}
}
