/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include "graphmatching/SeqStm.hpp"

using namespace Aidge; 




    ///////////////////////////////////////////////////////

    SeqStm::SeqStm( 
        const int stmIdx,
        const std::vector<std::vector<int>>& transitionMatrix,
        const std::map<std::string,NodeRegex*>& nodesRegex,
        const std::map<NodeTypeKey,int>& typeToIdxTransition,
        int actSt, 
        std::set<NodeTmp> allNodeValidated,
        std::set<NodeTmp> allNodeTested,
        std::set<std::pair<NodeTmp,std::string>> allCommonNode,
        bool stmIsValid):mStmIdx(stmIdx),
        mTransitionMatrix(transitionMatrix),
        mNodesRegex(nodesRegex),
        mTypeToIdxTransition(typeToIdxTransition)
        {

        //assert
        if (transitionMatrix.size() == 0){
            throw std::runtime_error ("no transitionMatrix");
        }
        if(transitionMatrix[0].size() == 0 || transitionMatrix[0].size() != typeToIdxTransition.size()){
            throw std::runtime_error ("bad transitionMatrix");
        }
        int size = static_cast<int>(transitionMatrix.size());
        if (actSt >= size){
            throw std::runtime_error ("bad actSt");
        }


        mActSt              = actSt;
        mAllNodeValidated   = allNodeValidated;
        mAllNodeTested      = allNodeTested;
        mAllCommonNode      = allCommonNode;
        mStmIsValid         = stmIsValid;

    }

    SeqStm* SeqStm::duplicateStm(){

        //deep copy of the set 
        // std::set<Node> cAllNodeValidated(mAllNodeValidated.begin(), mAllNodeValidated.end());
        // std::set<Node> cAllNodeTested(mAllNodeTested.begin(), mAllNodeTested.end());

        // std::set<std::pair<Node,std::string>> cAllCommonNode;
        // for (const auto& p : mAllCommonNode) {
        //     cAllCommonNode.insert(p);
        // }

        auto newStm = new SeqStm(
            mStmIdx,
            mTransitionMatrix,
            mNodesRegex,
            mTypeToIdxTransition,
            mActSt,
            mAllNodeValidated,
            mAllNodeTested,
            mAllCommonNode,
            mStmIsValid
        );

        return newStm;
    }


    std::pair<NodeRegex*,std::string> SeqStm::getNodeRegexAndCommonAt(int idxType)
    {
        //std::cout << "!" << idxType << "\n";
        for (auto const& x : mTypeToIdxTransition)
        {
            //x.second is the value : idx in mTransitionMatrix for the type
            //x.first pair of the node regex class and a string that is the common tag '',#,#n
            if (x.second == idxType ){

                if (mNodesRegex.find(x.first.first) != mNodesRegex.end()){
                    return std::make_pair(mNodesRegex.find(x.first.first)->second, x.first.second);  
                }else{
                    throw std::runtime_error ("a type is not define in NodesRegex");
                }
            }
        }
        throw std::runtime_error ("bad idx in mNodesRegex");
        return std::make_pair(nullptr,nullptr);
    }


    NodeType SeqStm::getTheNodeType(NodeTmp node)
    {
        //the node is a str of '{type}{idx}' and we juste want type 
        // // std::regex re("([a-zA-Z]+)[0-9]+");
        // // std::smatch match;
        // // if (std::regex_search(node, match, re) == true) {
        // //     return match.str(1);
        // // }
        // // throw std::runtime_error ("Type node not found");
        // // return "";

        //return node->name();
        return node->type();
    }


    std::string SeqStm::transitionOnNodeType(NodeType nodeType){

        if (!isStmBlocked()){
            int idxType = 0;
            for (auto & nextSt : mTransitionMatrix[mActSt]) {
                // There are a next step for this type
                //std::cout << "transition matrix next state -> "<< nextSt<<"\n" ;
                if (nextSt != -1){
                    //std::cout << "next -> "<< nextSt<< " "<< isAValidSt(nextSt) <<"\n" ;
                    auto nodeRegex = getNodeRegexAndCommonAt(idxType);
                    //std::cout << "-> "<< nodeRegex.second<<"\n" ;
                    if (nodeRegex.first->isA(nodeType)){
                        //std::cout << "nodetype tested !"<<"\n" ;
                        if(isAValidSt(nextSt)){
                            //std::cout << "Valid state !"<<"\n" ;
                            mStmIsValid = true;
                        }
                        mActSt = nextSt;
                        return nodeRegex.second;
                    }
                    
                }
                idxType += 1;
            }

            mActSt =-1;
        }

        return "";
    }


    std::pair<int,std::string> SeqStm::testNode(const NodeTmp node){
        
        std::string commonTag = "";
        //std::cout << "0\n" ;
        if (!isStmBlocked()){
            bool isNextStEnd = std::all_of(mTransitionMatrix[mActSt].begin(), mTransitionMatrix[mActSt].end(), [&](int x){ return x == -1; });
            //std::cout << "1:"<< isNextStEnd <<"\n" ;
            //if the next state if full of -1 can we relay add the node test to all node tested 
            // oker y test it but it sure that not be valid 
            if(!isNextStEnd){
                mAllNodeTested.insert(node);
            }
            //std::cout << "2\n" ;
            //recurtion avoidance
            if(mAllNodeValidated.find(node) == mAllNodeValidated.end()){
                
                NodeType nodeType = getTheNodeType(node);
                //std::cout << "3 " << nodeType << "\n" ;
                commonTag = transitionOnNodeType(nodeType);
                //after the transition test, if the node is != -1 the node is valid for the stm
                //std::cout << " mActSt = " << mActSt << "\n" ;
                if( mActSt != -1 ){
                    mAllNodeValidated.insert(node);
                }
            }else{
                mActSt = -1;
            }
        }

        if(commonTag != ""){
            mAllCommonNode.insert(std::make_pair(node,commonTag));
        }
        return std::make_pair(mActSt,commonTag);
    }


void SeqStm::drawStm(){

    //mTransitionMatrix
 // Find the maximum width of each column
    std::vector<std::size_t> max_widths(mTransitionMatrix[0].size(), 0);
    for (std::size_t i = 0; i < mTransitionMatrix.size(); ++i)
    {
        for (std::size_t j = 0; j < mTransitionMatrix[i].size(); ++j)
        {
            std::size_t width = std::to_string(mTransitionMatrix[i][j]).length();
            if (width > max_widths[j])
            {
                max_widths[j] = width;
            }
        }
    }

    // Print the vector with aligned columns
    for (std::size_t i = 0; i < mTransitionMatrix.size(); ++i)
    {
        for (std::size_t j = 0; j < mTransitionMatrix[i].size(); ++j)
        {
            int i_int = static_cast<int>(i);
            if (mActSt == -1 ){
                if(mStmIsValid){
                    std::cout << "\033[48;5;40m";
                }else{
                     std::cout << "\033[48;5;9m";
                }
            }
            else if (mActSt == i_int){
                std::cout << "\033[48;5;30m";
            }else{
                std::cout << "\033[48;5;27m";
            }

            // Pad the value with spaces to align it with the maximum width
            std::size_t width = std::to_string(mTransitionMatrix[i][j]).length();
            std::string padding(max_widths[j] - width, ' ');
            std::cout << padding << mTransitionMatrix[i][j] << " ";
            std::cout << "\033[0m";
        }
        std::cout << "\n";
    }

    std::cout << "mAllNodeTested : ";
    for (const auto& x : mAllNodeTested) {
        std::cout << x << ", ";
    }
    std::cout << "\n";


    std::cout << "mAllNodeValidated : ";
    for (const auto& x : mAllNodeValidated) {
        std::cout << x << ", ";
    }
    std::cout << "\n";
}

