#include "nodeTester/ConditionalInterpreter.hpp"
using namespace Aidge; 


///////////////////////////////
//ConditionalRegisterFunction
///////////////////////////////

    ConditionalData* ConditionalRegisterFunction::run(const std::string key,std::vector<ConditionalData*> & datas){

        auto lambdaIt = mWlambda.find(key);
        if (lambdaIt != mWlambda.end()) {
            return lambdaIt->second(datas);
        }else {
            throw std::runtime_error("can not run Lambda due to invalid key: " + key);
        }
    }

//////////////////////
//ConditionalInterpreter
///////////////////////
    ConditionalInterpreter::ConditionalInterpreter(const std::string ConditionalExpressions)
    :mLambdaRegiter()
    {

        ConditionalParser conditionalParser = ConditionalParser(ConditionalExpressions);
        mTree = conditionalParser.parse();
        ///lambda by default
        mLambdaRegiter.insert("getType",+[](NodePtr NodeOp){return NodeOp->type();});

    }
    

    bool ConditionalInterpreter::test( const NodePtr nodeOp)
    {
        try{
            std::vector<ConditionalData*> r =  visit({mTree},nodeOp);
   
        if (r.size() != 1){
            throw std::runtime_error("Multy output interpretation output");
        }else{
            if (!r[0]->isTypeEqualTo<bool>()){
                throw std::runtime_error("TEST OUT MUST BE A BOOL ");
            }else{
                return r[0]->getValue<bool>();
            }
        }

        }catch(const std::exception& e){
            std::ostringstream errorMessage;
            errorMessage << "Error in test " << "\n\t" << e.what()  << "\n";
            throw std::runtime_error(errorMessage.str()); 
        }

    }

    void ConditionalInterpreter::insertLambda(const std::string key,std::function<bool(Aidge::NodePtr)> f){
        mLambdaRegiter.insert<std::function<bool(Aidge::NodePtr)> >(key, f);
    }

    /////
    std::vector<ConditionalData*> ConditionalInterpreter::visit(const ASTNodeCh& nodes, const NodePtr nodeOp ){
            std::vector<ConditionalData*> dataVector;

            for ( std::shared_ptr<AstConditionalNode> node : nodes) {
                try{
                    switch (node->getType()){
                        ///////////////////////////////////
                        //OPERATOR
                        /////////////////////////////////// 
                        case ConditionalTokenTypes::NOT:
                            {
                            std::vector<ConditionalData*> tmp = fNot(visit(node->getChilds(),nodeOp));
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                        case ConditionalTokenTypes::AND:
                            {
                            std::vector<ConditionalData*> tmp = fAnd(visit(node->getChilds(),nodeOp));
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                        case ConditionalTokenTypes::OR:
                            {
                            std::vector<ConditionalData*> tmp = fOr(visit(node->getChilds(),nodeOp));
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                        case ConditionalTokenTypes::EQ:
                            {
                            std::vector<ConditionalData*> tmp = fEq(visit(node->getChilds(),nodeOp));
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                        case ConditionalTokenTypes::NEQ:
                            {
                            std::vector<ConditionalData*> tmp = fNeq(visit(node->getChilds(),nodeOp));
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;

                        ///////////////////////////////////
                        //VALUE
                        ///////////////////////////////////    
                    
                        case ConditionalTokenTypes::KEY:
                            
                            break;
                        case ConditionalTokenTypes::INTEGER:
                            {
                            std::vector<ConditionalData*> tmp = fStrToInteger(node);
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                        case ConditionalTokenTypes::FLOAT:
                            {
                            std::vector<ConditionalData*> tmp = fStrToFloat(node);
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                        case ConditionalTokenTypes::STRING:
                            {
                            std::vector<ConditionalData*> tmp = fStrToStr(node);
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;
                    
                        case ConditionalTokenTypes::NODE: //TODO
                            {

                            ConditionalData* data = new ConditionalData;
                            data->setValue<NodePtr>(nodeOp);
                            std::vector<ConditionalData*> tmp = {data};
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());

                            }
                            break;

                        case ConditionalTokenTypes::LAMBDA:
                            {
                            std::vector<ConditionalData*> tmp = fLambda(node,visit(node->getChilds(),nodeOp));
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;

                        case ConditionalTokenTypes::BOOL: //TODO
                            {
                            ConditionalData* data = new ConditionalData;
                            
                            if(node->getValue() == "true"){
                                data->setValue<bool>(true);
                            }else{
                                data->setValue<bool>(false);
                            }
                            
                            std::vector<ConditionalData*> tmp = {data};
                            dataVector.insert(dataVector.end(), tmp.begin(), tmp.end());
                            }
                            break;

                        case ConditionalTokenTypes::ARGSEP:
                        case ConditionalTokenTypes::LPAREN:
                        case ConditionalTokenTypes::RPAREN:
                        case ConditionalTokenTypes::STOP:
                        default:
                            throw std::runtime_error("NODE TYPE NOT SUPORTED IN ConditionalInterpreter");
                    }
                }catch(const std::exception& e){
                    std::ostringstream errorMessage;
                    errorMessage << "Error in visiting AST for node"<< nodeOp->name() << "\n\t" << e.what()  << "\n";
                    throw std::runtime_error(errorMessage.str()); 
                }
            }

            return dataVector;
    }


    //////////////////////
    //value convertor
    /////////////////////


    std::vector<ConditionalData*> ConditionalInterpreter::fStrToInteger(const std::shared_ptr<AstConditionalNode>& node)
    {
        std::vector<ConditionalData*> dataVector;
        ConditionalData* data = new ConditionalData;
        data->setValue<int>(std::stoi(node->getValue()));
        dataVector.push_back(data);
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fStrToFloat(const std::shared_ptr<AstConditionalNode>& node)
    {
        std::vector<ConditionalData*> dataVector;
        ConditionalData* data = new ConditionalData;
        data->setValue<float>(std::stof(node->getValue()));
        dataVector.push_back(data);
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fStrToStr(const std::shared_ptr<AstConditionalNode>& node)
    {
        std::vector<ConditionalData*> dataVector;
        ConditionalData* data = new ConditionalData;
        data->setValue<std::string>(node->getValue());
        dataVector.push_back(data);
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fLambda(const std::shared_ptr<AstConditionalNode>& node,std::vector<ConditionalData*> datas) 
    {
        //if the lambda have input
        std::vector<ConditionalData*> dataVector;
        ConditionalData* data;
        try {
            data = mLambdaRegiter.run(node->getValue(),datas);
        } catch (const std::exception& e) {
            std::ostringstream errorMessage;
            errorMessage << "Error in conditional interpretation when run the "<<  node->getValue() <<" Lambda\n\t" << e.what()  << "\n";
            throw std::runtime_error(errorMessage.str()); 
        }
        dataVector.push_back(data);
        datas.clear();
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fEq(std::vector<ConditionalData*> datas)
    {
        if (datas.size() != 2){
            throw std::runtime_error("EQ need 2 arg and get :" + datas.size());
        }
        auto a = datas[0];
        auto b = datas[1];

        if (a->getType() != b->getType()){
            throw std::runtime_error("EQ Unsuported between type :" + a->getType() +" "+ b->getType());
        }

        std::vector<ConditionalData*> dataVector;

        ConditionalData* data = new ConditionalData;

        if (a->isTypeEqualTo<int>()) {
           data->setValue<bool>( a->getValue<int>() == b->getValue<int>());
        }else if (a->isTypeEqualTo<float>()){
           data->setValue<bool>( a->getValue<float>() == b->getValue<float>());
        }else if (a->isTypeEqualTo<std::string>()){
           data->setValue<bool>( a->getValue<std::string>() == b->getValue<std::string>());
        }else if (a->isTypeEqualTo<bool>()){
           data->setValue<bool>( a->getValue<bool>() == b->getValue<bool>());
        }else{
           throw std::runtime_error("EQ Unknown type encountered :" + a->getType() );
        }
        dataVector.push_back(data);
        datas.clear();
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fNeq(std::vector<ConditionalData*> datas)
    {
        if (datas.size() != 2){
             throw std::runtime_error("NEQ need 2 arg and get :" + datas.size());
        }
        auto a = datas[0];
        auto b = datas[1];

        if (a->getType() != b->getType()){
            throw std::runtime_error("NEQ Unsuported between type :" + a->getType() +" "+ b->getType());
        }

        std::vector<ConditionalData*> dataVector;

        ConditionalData* data = new ConditionalData;

        if (a->isTypeEqualTo<int>()) {
           data->setValue<bool>( a->getValue<int>() != b->getValue<int>());
        }else if (a->isTypeEqualTo<float>()){
           data->setValue<bool>( a->getValue<float>() != b->getValue<float>());
        }else if (a->isTypeEqualTo<std::string>()){
           data->setValue<bool>( a->getValue<std::string>() != b->getValue<std::string>());
        }else
        {
           throw std::runtime_error("NEQ Unknown type encountered :" + a->getType() );
        }
        dataVector.push_back(data);
        datas.clear();
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fAnd(std::vector<ConditionalData*> datas)
    {
        if (datas.size() != 2){
           throw std::runtime_error("AND need 2 arg and get :" + datas.size());
        }
        auto a = datas[0];
        auto b = datas[1];

        std::vector<ConditionalData*> dataVector;

        if (a->getType() != typeid(bool).name() || b->getType() != typeid(bool).name()){
            throw std::runtime_error("AND Unknown type encountered need bool get :" + a->getType() );
        }

        ConditionalData* data = new ConditionalData;
        data->setValue<bool>( a->getValue<bool>() && b->getValue<bool>());
        

        dataVector.push_back(data);
        datas.clear();
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fOr(std::vector<ConditionalData*> datas)
    {
        if (datas.size() != 2){
             throw std::runtime_error("OR need 2 arg and get :" + datas.size());
        }
        auto a = datas[0];
        auto b = datas[1];

        std::vector<ConditionalData*> dataVector;

        if (a->getType() != typeid(bool).name() || b->getType() != typeid(bool).name()){
             throw std::runtime_error("OR Unknown type encountered need bool get :" + a->getType() );
        }

        ConditionalData* data = new ConditionalData;
        data->setValue<bool>( a->getValue<bool>() || b->getValue<bool>());
        

        dataVector.push_back(data);
        datas.clear();
        return dataVector;
    }

    std::vector<ConditionalData*> ConditionalInterpreter::fNot(std::vector<ConditionalData*> datas)
        {
            if (datas.size() != 1){
                throw std::runtime_error("not need 1 arg and get :" + datas.size());
            }
            auto a = datas[0];

            std::vector<ConditionalData*> dataVector;

            if (a->getType() != typeid(bool).name()){
                throw std::runtime_error("NOT Unknown type encountered need bool get :" + a->getType() );
            }

            ConditionalData* data = new ConditionalData;
            data->setValue<bool>( !a->getValue<bool>() );
            

            dataVector.push_back(data);
            datas.clear();
            return dataVector;
        }
