# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import subprocess
import sys
import pathlib
sys.path.insert(0, os.path.abspath('../python'))


# -- Project information -----------------------------------------------------

project = u'Eclipse Aidge'
copyright = u'2023, CEA LIST'
author = u'CEA LIST'


aidge_root = pathlib.Path().absolute()
version_file = open(aidge_root / "../version.txt", "r").read().strip()

# The short X.Y version
version = version_file
# The full version, including alpha/beta/rc tags
release = u''
# Name variable to use in the project
# /!\ name must be changed manually when it appears in code
rst_epilog = '.. |project_name| replace:: %s' % project


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
needs_sphinx = '1.8.3'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    #'sphinx_rtd_theme',
    'sphinx.ext.todo',
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.doctest',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.graphviz',
    'sphinxcontrib.bibtex',
    'breathe',
    'sphinxcontrib.mermaid',
    'sphinx_toolbox.collapse', # assume we have sphinx >= 1.8.0
    # "sphinx_gallery.gen_gallery",
    "sphinx_design",
    # 'myst_parser',
    'sphinx_copybutton'
]

graphviz_output_format = 'svg'

# Mermaid Configuration
mermaid_version = "9.3.0"

# Breathe Configuration


# Doxygen
subprocess.call('doxygen Doxyfile.in', shell=True)
breathe_projects = {
	"aidge": "xml/"
}
breathe_default_project = "aidge"
breathe_default_members = ('members', 'undoc-members')

#Sphinx-gallery config
# sphinx_gallery_conf = {
#      'examples_dirs': 'Tutorial/examples',   # path to your example scripts
#      'gallery_dirs': 'Tutorial/auto_examples',  # path to where to save gallery generated output
# }

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']
# generate autosummary pages
autosummary_generate = True

# todo config
todo_include_todos = True
todo_emit_warnings = True

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
source_suffix = ['.rst', '.md']
# source_suffix = '.rst'

# The master toctree document.
master_doc = 'source/index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [u'_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = None


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
# #
# if os.environ.get('READTHEDOCS') != 'True':
#     try:
#         import sphinx_rtd_theme
#     except ImportError:
#         pass  # assume we have sphinx >= 1.3
#     else:
#         html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

#     html_theme = 'sphinx_rtd_theme'


# if os.environ.get('READTHEDOCS') != 'True':
#     try:
#         import pydata_sphinx_theme
#     except ImportError:
#         pass  # assume we have sphinx >= 1.3
#     else:
#         html_theme_path = [pydata_sphinx_theme.get_html_theme_path()]

#     html_theme = 'pydata_sphinx_theme'

html_theme = 'pydata_sphinx_theme'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}
html_theme_options = {
    # 'logo_only': True,
    # 'style_nav_header_background': '#F5F5F5',
    "show_toc_level": 1,
    "show_nav_level": 2,
    "navigation_depth": 2,
    "navbar_align": "content",
    "header_links_before_dropdown": 7,
    "use_edit_page_button": True,
    # "logo": {
    #     "text": project,
    #     "image_dark": "figures/Aidge.png",
    #     "alt_text": project,
    # },
    "icon_links": [
        {
            "name": "GitLab",
            "url": "<https://gitlab.eclipse.org/eclipse/aidge>",
            "icon": "fab fa-gitlab",
        }
    ],
     "announcement":"🚧 Work in Progress 🚧"
}

html_context = {
    "gitlab_url": "https://gitlab.eclipse.org/eclipse/aidge", # or your self-hosted GitLab
    "gitlab_user": "Aidge",
    "gitlab_repo": "refactorideas",
    "gitlab_version": "master",
    "doc_path": "source",
}


# html_logo = '' # TODO : Add logo path

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['source/_static']

bibtex_bibfiles = [html_static_path[0] + '/refs.bib']

html_css_files = [
    'css/custom.css',
]

html_js_files = [
   'js/mermaid.min.js'
]

json_url = html_static_path[0] + '/switcher.json'

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself.  Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for HTMLHelp output ---------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'Aidgedoc'


# -- Options for LaTeX output ------------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    'preamble': r'''\usepackage{listings}
        \usepackage{pifont}
        \usepackage{newunicodechar}
        \newunicodechar{✓}{\ding{51}}
        \newunicodechar{✗}{\ding{55}}
        \setcounter{tocdepth}{3}''',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'Aidge.tex', u'Aidge Documentation',
     u'CEA LIST', 'manual'),
]


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'Aidge', u'Aidge Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'Aidge', u'Aidge Documentation',
     author, 'Aidge', 'One line description of project.',
     'Miscellaneous'),
]


# -- Options for Epub output -------------------------------------------------

# Bibliographic Dublin Core info.
epub_title = project

# The unique identifier of the text. This can be a ISBN number
# or the project homepage.
#
# epub_identifier = ''

# A unique identification for the text.
#
# epub_uid = ''

# A list of files that should not be packed into the epub file.
epub_exclude_files = ['search.html']

# __init__ is always documented
def skip(app, what, name, obj, would_skip, options):
    if name == "__init__":
        return False
    return would_skip

def setup(app):
    app.connect("autodoc-skip-member", skip)
# -- Extension configuration -------------------------------------------------
